/*
SpeciesChecker
by Cory McKay

CLASS NAME:		AnalyzerGeneral
CLASS TYPE:		Analysis rules
DESCRIPTION:	Contains methods that are used to check if individual rules relating to the overall
				rules of a species piece have been violated. There is one method for each rule.
				AnalyzerGeneral objects should be eliminated after the analysis has been
				completed.
*/


class AnalyzerGeneral
{
	/* FIELDS ******************************************************************/


	private ErrorList errors_found;	// the list of errors found
	private int time_signature;		//  the time signature of the cantus
	private int number_of_bars;		// the total number of bars in the cantus

	
	/* CONSTRUCTOR *************************************************************/


	AnalyzerGeneral(ErrorList err, int tim, int num)
	{
		errors_found = err;
		time_signature = tim;
		number_of_bars = num;
	}


	/* PUBLIC METHODS **********************************************************/


	public void check_meter()
	// Check to see if the following rule is violated:
	// The piece must have a time signature of 4/4.
	{
		if (time_signature != 16)
			errors_found.insert_new_error("ERROR: The piece has an incorrect meter." + 
				" It should have a time signature of 4/4.");
	}


	/***************************************************************************/


	public void check_length()
	// Check to see if the following rule is violated:
	// The piece should be between eight and sixteen measures long.
	{
		if (number_of_bars < 8)
			errors_found.insert_new_error("ERROR: The piece is too short." +
				" It should generally be between eight and sixteen measures long.");
		if (number_of_bars > 16)
			errors_found.insert_new_error("WARNING: The piece is too long." +
				" It should generally be between eight and sixteen measures long.");
	}
}