/*
SpeciesChecker
by Cory McKay

CLASS NAME:		ErrorList
CLASS TYPE:		Analysis data structure
DESCRIPTION:	Objects of this class represent nodes in a list of errors found by analysis classes.
				Appropriate for use in species counterpoint and simple harmony.
*/


class ErrorList 
{
	/* FIELDS ******************************************************************/


	private String current_error;				// the error recorded the current node
	private ErrorList previous_node;			// the previous node in the list. Is null when there are no nodes
												// before the current node.
	private static ErrorList front_of_queue;	// the item that was first inserted into the queue
	private static ErrorList end_of_queue;		// the item that has most recently been inserted into the queue


	/* CONSTRUCTORS ************************************************************/


	ErrorList()
	// Creates an empty node. Should be used to create the first node.
	{
		current_error = null;
		previous_node = null;
		front_of_queue = this;
		end_of_queue = this;
	}


	/***************************************************************************/


	ErrorList(String error_contents, ErrorList old_node)
	// Creates a new node that contains the error error_contents.
	// Should be used to create all but the first node.
	{
		current_error = error_contents;
		previous_node = null;
		old_node.previous_node = this;
		end_of_queue = this;
	}


	/* PUBLIC METHODS **********************************************************/


	public static void insert_new_error(String contents)
	// Inserts a node at the end of the list with the specified contents.
	// If there is only one node and it is blank, then gives it the specified contents instead.
	{
		if (front_of_queue.current_error == null)
			front_of_queue.current_error = contents;
		else if (front_of_queue.previous_node != null)
			end_of_queue = new ErrorList(contents, end_of_queue);
		else
			end_of_queue = new ErrorList(contents, front_of_queue);
	}


	/***************************************************************************/


	public String get_error()
	// Returns the error at the front of the queue. Deletes this node.
	{
		String an_error = null;
		if (front_of_queue.current_error != null)
		{
			an_error = front_of_queue.current_error;
			remove_node();
		}
		return an_error;
	}


	/***************************************************************************/


	public static int is_there_an_error()
	// Returns 1 if errors are found, returns 0 if none are found
	{
		int error_found = 0; // the number of errors that are found
		while (front_of_queue.current_error != null)
		{
			remove_node();
			error_found++;
		}

		if (error_found == 0)
			return 0;
		else
			return 1;
	}


	/***************************************************************************/


	public static void print_error_list()
	// Prints out the list of errors found to standard out, from the first one found
	// to the last one found. Deletes nodes as this is done.
	{
		int error_found = 0; // the number of errors that are found
		while (front_of_queue.current_error != null)
		{
			System.out.println();
			System.out.println(front_of_queue.current_error);
			remove_node();
			error_found++;
		}

		if (error_found == 0)
		{
			System.out.println();
			System.out.println("No errors detected.");
		}
	}


	/* PRIVATE METHODS *********************************************************/


	private static String remove_node()
	// Returns the error at the front of the list.
	// Deletes the node if there is another node or sets the contents to null if there is not.
	{
		String return_string = front_of_queue.current_error;
		if (front_of_queue.previous_node != null)
			front_of_queue = front_of_queue.previous_node;
		else
			front_of_queue.current_error = null;
		return return_string;
	}
}