/*
SpeciesChecker
by Cory McKay

CLASS NAME:		HelpDialog
CLASS TYPE:		GUI - dialog box
DESCRIPTION:	A dialog box that displays html files on how to use the SpeciesChecker program.
				The index html file must be in a directory called Help and it must be called index.html.
*/


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;


class HelpDialog extends JDialog
{
	/* FIELDS ******************************************************************/


	JEditorPane displayPane; // where the actual help data is displayed


	/* CONSTRUCTOR *************************************************************/


	public HelpDialog()
	{
		// Give the dialog box its owner, its size, its title and make it non-modal
		super();
		setTitle("Help");
		setModal(false);
		setSize(630, 572);


		// Set up the display panel and make it load the index page
		displayPane = new JEditorPane();
		displayPane.setEditable(false);
		reset_display();


		// Enable the loading of other help pages
		displayPane.addHyperlinkListener(new HyperlinkListener()
		{
			public void	hyperlinkUpdate(HyperlinkEvent event)
			{
				if (event.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
				{
					try	{displayPane.setPage(event.getURL());}
					catch (Exception f) {displayPane.setText("Unable to find requested help page.");}
				}
			}
		});


		// Set up an Index button to go to the index page
		JButton index = new JButton("Index");
		index.addActionListener(new
		ActionListener()
		{
			public void actionPerformed(ActionEvent evt)
			{
				reset_display();;
			}
		});


		// Set up an OK button to close the dialog box
		JButton ok = new JButton("OK");
		ok.addActionListener(new
		ActionListener()
		{
			public void actionPerformed(ActionEvent evt)
			{
				setVisible(false);
			}
		});


		// Display the screen components
		Container contentPane = getContentPane();
		contentPane.add(new JScrollPane(displayPane), BorderLayout.CENTER);
		JPanel panel = new JPanel();
		panel.add(index);
		panel.add(ok);
		contentPane.add(panel, BorderLayout.SOUTH);
	}


	/* PRIVATE METHODS *********************************************************/


	private void reset_display()
	// Display the default help page
	{
		try {displayPane.setPage("file:Help/index.html");}
		catch (Exception e) {displayPane.setText("Unable to find the main help page.");}
	}
}