/*
SpeciesChecker
by Cory McKay

CLASS NAME:		MusicTranslator
CLASS TYPE:		Data format translation
DESCRIPTION:	Used to translate encoded musical data into a human readable format.
				Appropriate for use in species counterpoint and simple harmony.
*/


class MusicTranslator
{
	/* PUBLIC METHODS **********************************************************/


	public String produceCompositinHeader(Composition piece, Voice cantus, Voice counterpoint)
	// Returns a string of text containing information about the type of composition, its key,
	// its time signature and its length. The formatting of this text is appropriate for the 
	// information area at the top of the SpeciesChecker GUI.
	{
		String type;
		if (piece.get_type_of_composition() == 0) type = new String("Cantus firmus only");
		else if (counterpoint.get_type_of_line() == 1) type = new String("First species, cantus lower voice");
		else if (counterpoint.get_type_of_line() == 2) type = new String("First species, cantus upper voice");
		else if (counterpoint.get_type_of_line() == 3) type = new String("Second species, cantus lower voice");
		else if (counterpoint.get_type_of_line() == 4) type = new String("Second species, cantus upper voice");
		else if (counterpoint.get_type_of_line() == 5) type = new String("Third species, cantus lower voice");
		else if (counterpoint.get_type_of_line() == 6) type = new String("Third species, cantus upper voice");
		else if (counterpoint.get_type_of_line() == 7) type = new String("Fourth species, cantus lower voice");
		else if (counterpoint.get_type_of_line() == 8) type = new String("Fourth species, cantus upper voice");
		else if (counterpoint.get_type_of_line() == 9) type = new String("Fifth species, cantus lower voice");
		else if (counterpoint.get_type_of_line() == 10) type = new String("Fifth species, cantus upper voice");
		else type = new String("Cantus firmus only");
		
		String key;
		if (piece.get_key_signature() == 1 && piece.get_quality() == 1) key = new String("C major");
		else if (piece.get_key_signature() == 1 && piece.get_quality() == 2) key = new String("A minor");
		else if (piece.get_key_signature() == 2 && piece.get_quality() == 1) key = new String("Db major");
		else if (piece.get_key_signature() == 2 && piece.get_quality() == 2) key = new String("Bb minor");
		else if (piece.get_key_signature() == 3 && piece.get_quality() == 1) key = new String("D major");
		else if (piece.get_key_signature() == 3 && piece.get_quality() == 2) key = new String("B minor");
		else if (piece.get_key_signature() == 4 && piece.get_quality() == 1) key = new String("Eb major");
		else if (piece.get_key_signature() == 4 && piece.get_quality() == 2) key = new String("C minor");
		else if (piece.get_key_signature() == 5 && piece.get_quality() == 1) key = new String("E major");
		else if (piece.get_key_signature() == 5 && piece.get_quality() == 2) key = new String("C# minor");
		else if (piece.get_key_signature() == 6 && piece.get_quality() == 1) key = new String("F major");
		else if (piece.get_key_signature() == 6 && piece.get_quality() == 2) key = new String("D minor");
		else if (piece.get_key_signature() == 7 && piece.get_quality() == 1) key = new String("Gb major");
		else if (piece.get_key_signature() == 7 && piece.get_quality() == 2) key = new String("Eb minor");
		else if (piece.get_key_signature() == 8 && piece.get_quality() == 1) key = new String("G major");
		else if (piece.get_key_signature() == 8 && piece.get_quality() == 2) key = new String("E minor");
		else if (piece.get_key_signature() == 9 && piece.get_quality() == 1) key = new String("Ab major");
		else if (piece.get_key_signature() == 9 && piece.get_quality() == 2) key = new String("F minor");
		else if (piece.get_key_signature() == 10 && piece.get_quality() == 1) key = new String("A major");
		else if (piece.get_key_signature() == 10 && piece.get_quality() == 2) key = new String("F# minor");
		else if (piece.get_key_signature() == 11 && piece.get_quality() == 1) key = new String("Bb major");
		else if (piece.get_key_signature() == 11 && piece.get_quality() == 2) key = new String("G minor");
		else if (piece.get_key_signature() == 12 && piece.get_quality() == 1) key = new String("B major");
		else if (piece.get_key_signature() == 12 && piece.get_quality() == 2) key = new String("G# minor");
		else key = new String("C major");

		String meter;
		if (piece.get_time_signature() == 16) meter = new String("4 / 4");
		else if (piece.get_time_signature() == 8) meter = new String("2 / 4");
		else if (piece.get_time_signature() == 12) meter = new String("3 / 4");
		else if (piece.get_time_signature() == 24) meter = new String("12 / 8");
		else meter = new String("4 / 4");

		int measures = piece.get_number_of_bars();
				
		String composition_header = "TYPE OF COMPOSITION: " + type + "\n" +
			"KEY OF COMPOSITION: " +  key + "\n" +
			"METER OF COMPOSITION: " + meter + "\n" +
			"LENGTH IN MEASURES: " + measures;
		return composition_header;
	}


	/***************************************************************************/


	public String[] get_line_information(Composition piece, int index)
	// Returns an array of strings containing information about the Voice at the given index of the Composition.
	// The format as is follows: type of line, key signature, time signature, clef, number of measures and
	// then the information about each note (measure, pitch, duration, tied).
	// Used mainly for debugging purposes.
	{
		// Determine the number of notes in the line and initialize line_description
		int number_of_notes= 0; // the number of notes in piece
		for (int i = 0; i < piece.get_total_sixteenth_notes(); i++)
		{
			if (piece.get_voice(index).get_note(i) != null)
				number_of_notes++;
		}
		String line_description[] = new String[5 + 4 * number_of_notes]; // the information to be returned
		int current_index = 0; // the current index of line_description

		// Record the initial information about the line
		int key_signature = piece.get_key_signature();
		int time_signature = piece.get_time_signature();
		int quality = piece.get_quality();
		line_description[0] = "Type of line: " + interperet_type_of_line(piece.get_voice(index).get_type_of_line());
		line_description[1] = "Key: " + interperet_key_signature(key_signature, quality);
		line_description[2] = "Time signature: " + interperet_time_signature(time_signature);
		line_description[3] = "Clef: " + interperet_clef(piece.get_voice(index).get_clef());
		line_description[4] = "Number of measures: " + String.valueOf(piece.get_number_of_bars());
		current_index = 5;


		// Record the information for each note
		for (int i = 0; i < piece.get_total_sixteenth_notes(); i++)
		{
			if (piece.get_voice(index).get_note(i) != null)
			{				
				// Store the measure number
				line_description[current_index] = String.valueOf(1 + i / time_signature);
				current_index++;

				// Store the note's pitch
				line_description[current_index] = interperet_pitch(piece.get_voice(index).get_note(i).get_pitch(),
					key_signature, piece.get_quality());
				current_index++;
				
				// Store the note's duration
				line_description[current_index] = interperet_duration(piece.get_voice(index).get_note(i).get_duration());
				current_index++;

				// Store whether the note is tied
				line_description[current_index] = interperet_tied(piece.get_voice(index).get_note(i).get_tied());
				current_index++;
			}
		}

		
		// Return the information
		return line_description;
	}


	/***************************************************************************/


	public void ouptut_line(String[] output_data)
	// Prints data of the type produced by the get_line_information method (see above) to standard out.
	// Used mainly for debugging purposes.
	{
		// Print header information
		System.out.println();
		for (int i = 0; i < 5; i ++)
		{
			System.out.println(output_data[i]);
		}

		
		// Print note table
		int string_length = 0; // the length of the string to be printed
		System.out.println();
		System.out.println("MEASURE  PITCH  DURATION            TIED");
		System.out.println("-------  -----  --------            -----");
		for (int i = 5; i < output_data.length; i++)
		{
			System.out.print(output_data[i]);
			add_spaces(output_data[i], 9);
			i++;
			System.out.print(output_data[i]);
			add_spaces(output_data[i], 7);
			i++;
			System.out.print(output_data[i]);
			add_spaces(output_data[i], 20);
			i++;
			System.out.print(output_data[i]);
			System.out.println();
		}
	}


	/***************************************************************************/


	public String interperet_time_signature(int time_signature)
	// Returns a string containing the name of the given time signature.
	{
		String time_signature_name;
		
		if (time_signature == 8) time_signature_name = new String("2/4");
		else if (time_signature == 12) time_signature_name = new String("3/4");
		else if (time_signature == 16) time_signature_name = new String("4/4");
		else if (time_signature == 24) time_signature_name = new String("12/8");
		else time_signature_name = new String("Invalid Time Signature");

		return time_signature_name;
	}


	/***************************************************************************/


	public String interperet_quality(int quality)
	// Returns a string indicating whether a line is major or minor.
	{
		String quality_name;
		
		if (quality == 1) quality_name = new String("Major");
		else if (quality == 2) quality_name = new String("Minor");
		else quality_name = new String("Invalid Quality");

		return quality_name;
	}


	/***************************************************************************/


	public String interperet_key_signature(int key_signature, int quality)
	// Returns a string containing the name of the given key signature.
	{
		String key_signature_name;

		// Major keys
		if (quality == 1)
		{
			if (key_signature == 1) key_signature_name = new String("C major");
			else if (key_signature == 2) key_signature_name = new String("Db major");
			else if (key_signature == 3) key_signature_name = new String("D major");
			else if (key_signature == 4) key_signature_name = new String("Eb major");
			else if (key_signature == 5) key_signature_name = new String("E major");
			else if (key_signature == 6) key_signature_name = new String("F major");
			else if (key_signature == 7) key_signature_name = new String("Gb major");
			else if (key_signature == 8) key_signature_name = new String("G major");
			else if (key_signature == 9) key_signature_name = new String("Ab major");
			else if (key_signature == 10) key_signature_name = new String("A major");
			else if (key_signature == 11) key_signature_name = new String("Bb major");
			else if (key_signature == 12) key_signature_name = new String("B major");
			else key_signature_name = new String("Invalid Key Signature");
		}

		// Minor keys
		else
		{
			if (key_signature == 1) key_signature_name = new String("A minor");
			else if (key_signature == 2) key_signature_name = new String("Bb minor");
			else if (key_signature == 3) key_signature_name = new String("B minor");
			else if (key_signature == 4) key_signature_name = new String("C minor");
			else if (key_signature == 5) key_signature_name = new String("C# minor");
			else if (key_signature == 6) key_signature_name = new String("D minor");
			else if (key_signature == 7) key_signature_name = new String("Eb minor");
			else if (key_signature == 8) key_signature_name = new String("E minor");
			else if (key_signature == 9) key_signature_name = new String("F minor");
			else if (key_signature == 10) key_signature_name = new String("F# minor");
			else if (key_signature == 11) key_signature_name = new String("G minor");
			else if (key_signature == 12) key_signature_name = new String("G# minor");
			else key_signature_name = new String("Invalid Key Signature");
		}

		return key_signature_name;
	}


	/***************************************************************************/


	public String interperet_type_of_line(int type_of_line)
	// Returns a string containing the given type of line of a voice
	{
		String type_of_line_name;

		if (type_of_line == 0) type_of_line_name = new String("Cantus firmus");
		else if (type_of_line == 0) type_of_line_name = new String("cantus firmus");
		else if (type_of_line == 1) type_of_line_name = new String("first species above cantus");
		else if (type_of_line == 2) type_of_line_name = new String("first species below cantus");
		else if (type_of_line == 3) type_of_line_name = new String("second species above cantus");
		else if (type_of_line == 4) type_of_line_name = new String("second species below cantus");
		else if (type_of_line == 5) type_of_line_name = new String("third species above cantus");
		else if (type_of_line == 6) type_of_line_name = new String("third species below cantus");
		else if (type_of_line == 7) type_of_line_name = new String("fourth species above cantus");
		else if (type_of_line == 8) type_of_line_name = new String("fourth species below cantus");
		else if (type_of_line == 9) type_of_line_name = new String("fifth species above cantus");
		else if (type_of_line == 10) type_of_line_name = new String("fifth species below cantus");
		else if (type_of_line == 11) type_of_line_name = new String("bass in four-part composition");
		else if (type_of_line == 12) type_of_line_name = new String("tenor in four-part composition");
		else if (type_of_line == 13) type_of_line_name = new String("alto in four-part composition");
		else if (type_of_line == 14) type_of_line_name = new String("soprano in four-part composition");
		else type_of_line_name = new String("Invalid Type of Line");

		return type_of_line_name;
	}


	/***************************************************************************/


	public String interperet_clef(int clef)
	// Returns a string containing the given clef of a voice
	{
		String clef_name;
		
		if (clef == 1) clef_name = new String("bass");
		else if (clef == 2) clef_name = new String("C");
		else if (clef == 3) clef_name = new String("alto");
		else clef_name = new String("Invalid Clef");

		return clef_name;
	}


	/***************************************************************************/


	public String interperet_pitch(int pitch, int key_signature, int quality)
	// Returns a string containing the name of the given pitch.
	{
		String pitch_name;
		int leading_tone = 0; // the raised seventh scale degree in a minor key


		// Find the leading tone in minor keys
		if (quality == 2 && pitch != 0)
		{
			if (key_signature == 1)
				leading_tone = 9;
			else if (key_signature == 2)
				leading_tone = 10;
			else if (key_signature == 3)
				leading_tone = 11;
			else if (key_signature == 4)
				leading_tone = 12;
			else
				leading_tone = key_signature - 4;
		}


		// Deal with raised leading tones in minor keys
		if (quality == 2 && leading_tone == pitch % 12 && pitch != 0)
		{
			int register;
			if (pitch < 13)
				register = -2;
			else if (pitch < 25)
				register = -1;
			else if (pitch < 37)
				register = 1;
			else if (pitch < 49)
				register = 2;
			else
				register = 3;

			if (key_signature == 1) pitch_name = new String("G# " + register);
			else if (key_signature == 2) pitch_name = new String("A  " + register);
			else if (key_signature == 3) pitch_name = new String("A# " + register);
			else if (key_signature == 4) pitch_name = new String("B  " + register);
			else if (key_signature == 5) pitch_name = new String("C  " + register);
			else if (key_signature == 6) pitch_name = new String("C# " + register);
			else if (key_signature == 7) pitch_name = new String("D  " + register);
			else if (key_signature == 8) pitch_name = new String("D# " + register);
			else if (key_signature == 9) pitch_name = new String("E  " + register);
			else if (key_signature == 10) pitch_name = new String("E# " + register);
			else if (key_signature == 11) pitch_name = new String("F# " + register);
			else if (key_signature == 12) pitch_name = new String("G  " + register);
			else pitch_name = new String("Invalid Pitch");
		}

		
		// Keys with sharps

		else if (key_signature == 1 || key_signature == 3 || key_signature == 5 || key_signature == 8 || 
			key_signature == 10 || key_signature == 12)
		{
			if (pitch == 0) pitch_name = new String("Rest");
			else if (pitch == 1) pitch_name = new String("C  -2");
			else if (pitch == 2) pitch_name = new String("C# -2");
			else if (pitch == 3) pitch_name = new String("D  -2");
			else if (pitch == 4) pitch_name = new String("D# -2");
			else if (pitch == 5) pitch_name = new String("E  -2");
			else if (pitch == 6) pitch_name = new String("F  -2");
			else if (pitch == 7) pitch_name = new String("F# -2");
			else if (pitch == 8) pitch_name = new String("G  -2");
			else if (pitch == 9) pitch_name = new String("G# -2");
			else if (pitch == 10) pitch_name = new String("A  -2");
			else if (pitch == 11) pitch_name = new String("A# -2");
			else if (pitch == 12) pitch_name = new String("B  -2");
			else if (pitch == 13) pitch_name = new String("C  -1");
			else if (pitch == 14) pitch_name = new String("C# -1");
			else if (pitch == 15) pitch_name = new String("D  -1");
			else if (pitch == 16) pitch_name = new String("D# -1");
			else if (pitch == 17) pitch_name = new String("E  -1");
			else if (pitch == 18) pitch_name = new String("F  -1");
			else if (pitch == 19) pitch_name = new String("F# -1");
			else if (pitch == 20) pitch_name = new String("G  -1");
			else if (pitch == 21) pitch_name = new String("G# -1");
			else if (pitch == 22) pitch_name = new String("A  -1");
			else if (pitch == 23) pitch_name = new String("A# -1");
			else if (pitch == 24) pitch_name = new String("B  -1");
			else if (pitch == 25) pitch_name = new String("C  1");
			else if (pitch == 26) pitch_name = new String("C# 1");
			else if (pitch == 27) pitch_name = new String("D  1");
			else if (pitch == 28) pitch_name = new String("D# 1");
			else if (pitch == 29) pitch_name = new String("E  1");
			else if (pitch == 30) pitch_name = new String("F  1");
			else if (pitch == 31) pitch_name = new String("F# 1");
			else if (pitch == 32) pitch_name = new String("G  1");
			else if (pitch == 33) pitch_name = new String("G# 1");
			else if (pitch == 34) pitch_name = new String("A  1");
			else if (pitch == 35) pitch_name = new String("A# 1");
			else if (pitch == 36) pitch_name = new String("B  1");
			else if (pitch == 37) pitch_name = new String("C  2");
			else if (pitch == 38) pitch_name = new String("C# 2");
			else if (pitch == 39) pitch_name = new String("D  2");
			else if (pitch == 40) pitch_name = new String("D# 2");
			else if (pitch == 41) pitch_name = new String("E  2");
			else if (pitch == 42) pitch_name = new String("F  2");
			else if (pitch == 43) pitch_name = new String("F# 2");
			else if (pitch == 44) pitch_name = new String("G  2");
			else if (pitch == 45) pitch_name = new String("G# 2");
			else if (pitch == 46) pitch_name = new String("A  2");
			else if (pitch == 47) pitch_name = new String("A# 2");
			else if (pitch == 48) pitch_name = new String("B  2");
			else if (pitch == 49) pitch_name = new String("C  3");
			else if (pitch == 50) pitch_name = new String("C# 3");
			else if (pitch == 51) pitch_name = new String("D  3");
			else if (pitch == 52) pitch_name = new String("D# 3");
			else if (pitch == 53) pitch_name = new String("E  3");
			else if (pitch == 54) pitch_name = new String("F  3");
			else if (pitch == 55) pitch_name = new String("F# 3");
			else if (pitch == 56) pitch_name = new String("G  3");
			else if (pitch == 57) pitch_name = new String("G# 3");
			else if (pitch == 58) pitch_name = new String("A  3");
			else if (pitch == 59) pitch_name = new String("A# 3");
			else if (pitch == 60) pitch_name = new String("B  3");
			else pitch_name = new String("Invalid Pitch");
		}


		// Keys with flats
		else
		{
			if (pitch == 0) pitch_name = new String("Rest");
			else if (pitch == 1) pitch_name = new String("C  -2");
			else if (pitch == 2) pitch_name = new String("Db -2");
			else if (pitch == 3) pitch_name = new String("D  -2");
			else if (pitch == 4) pitch_name = new String("Eb -2");
			else if (pitch == 5) pitch_name = new String("E  -2");
			else if (pitch == 6) pitch_name = new String("F  -2");
			else if (pitch == 7) pitch_name = new String("Gb -2");
			else if (pitch == 8) pitch_name = new String("G  -2");
			else if (pitch == 9) pitch_name = new String("Ab -2");
			else if (pitch == 10) pitch_name = new String("A  -2");
			else if (pitch == 11) pitch_name = new String("Bb -2");
			else if (pitch == 12) pitch_name = new String("B  -2");
			else if (pitch == 13) pitch_name = new String("C  -1");
			else if (pitch == 14) pitch_name = new String("Db -1");
			else if (pitch == 15) pitch_name = new String("D  -1");
			else if (pitch == 16) pitch_name = new String("Eb -1");
			else if (pitch == 17) pitch_name = new String("E  -1");
			else if (pitch == 18) pitch_name = new String("F  -1");
			else if (pitch == 19) pitch_name = new String("Gb -1");
			else if (pitch == 20) pitch_name = new String("G  -1");
			else if (pitch == 21) pitch_name = new String("Ab -1");
			else if (pitch == 22) pitch_name = new String("A  -1");
			else if (pitch == 23) pitch_name = new String("Bb -1");
			else if (pitch == 24) pitch_name = new String("B  -1");
			else if (pitch == 25) pitch_name = new String("C  1");
			else if (pitch == 26) pitch_name = new String("Db 1");
			else if (pitch == 27) pitch_name = new String("D  1");
			else if (pitch == 28) pitch_name = new String("Eb 1");
			else if (pitch == 29) pitch_name = new String("E  1");
			else if (pitch == 30) pitch_name = new String("F  1");
			else if (pitch == 31) pitch_name = new String("Gb 1");
			else if (pitch == 32) pitch_name = new String("G  1");
			else if (pitch == 33) pitch_name = new String("Ab 1");
			else if (pitch == 34) pitch_name = new String("A  1");
			else if (pitch == 35) pitch_name = new String("Bb 1");
			else if (pitch == 36) pitch_name = new String("B  1");
			else if (pitch == 37) pitch_name = new String("C  2");
			else if (pitch == 38) pitch_name = new String("Db 2");
			else if (pitch == 39) pitch_name = new String("D  2");
			else if (pitch == 40) pitch_name = new String("Eb 2");
			else if (pitch == 41) pitch_name = new String("E  2");
			else if (pitch == 42) pitch_name = new String("F  2");
			else if (pitch == 43) pitch_name = new String("Gb 2");
			else if (pitch == 44) pitch_name = new String("G  2");
			else if (pitch == 45) pitch_name = new String("Ab 2");
			else if (pitch == 46) pitch_name = new String("A  2");
			else if (pitch == 47) pitch_name = new String("Bb 2");
			else if (pitch == 48) pitch_name = new String("B  2");
			else if (pitch == 49) pitch_name = new String("C  3");
			else if (pitch == 50) pitch_name = new String("Db 3");
			else if (pitch == 51) pitch_name = new String("D  3");
			else if (pitch == 52) pitch_name = new String("Eb 3");
			else if (pitch == 53) pitch_name = new String("E  3");
			else if (pitch == 54) pitch_name = new String("F  3");
			else if (pitch == 55) pitch_name = new String("Gb 3");
			else if (pitch == 56) pitch_name = new String("G  3");
			else if (pitch == 57) pitch_name = new String("Ab 3");
			else if (pitch == 58) pitch_name = new String("A  3");
			else if (pitch == 59) pitch_name = new String("Bb 3");
			else if (pitch == 60) pitch_name = new String("B  3");
			else pitch_name = new String("Invalid Pitch");
		}
		
		return pitch_name;
	}


	/***************************************************************************/


	public String interperet_duration(int duration)
	// Returns a string with the given duration of a Note
	{
		String duration_name;

		if (duration == 1) duration_name = new String("Sixteenth note");
		else if (duration == 2) duration_name = new String("Eighth note");
		else if (duration == 3) duration_name = new String("Dotted 8th note");
		else if (duration == 4) duration_name = new String("Quater note");
		else if (duration == 6) duration_name = new String("Dotted quarter note");
		else if (duration == 8) duration_name = new String("Half note");
		else if (duration == 12) duration_name = new String("Dotted half note");
		else if (duration == 16) duration_name = new String("Whole note");
		else duration_name = new String("Invalid note duration");

		return duration_name;
	}


	/***************************************************************************/


	public String interperet_tied(int tied)
	// Returns a string with the tied status of a Note
	{
		String tied_name;

		if (tied == 1) tied_name = new String("Not tied");
		else if (tied == 2) tied_name = new String("Tied back");
		else if (tied == 3) tied_name = new String("Tied ahead");
		else if (tied == 4) tied_name = new String("Tied both directions");
		else tied_name = new String("Invalid tied value");

		return tied_name;
	}

	
	/* PRIVATE METHODS *********************************************************/


	private void add_spaces(String entry, int available_spaces)
	// Prints the number of spaces needed to fill a table column after entry is printed
	// Used by the output_line method (see above).
	{
		for (int i = 0; i < available_spaces - entry.length(); i++)
		{
			System.out.print(" ");
		}
	}
}