/*
SpeciesChecker
by Cory McKay

CLASS NAME:		Note
CLASS TYPE:		Fundamental data structure
DESCRIPTION:	Objects of this class represent single notes.
				Appropriate for use in species counterpoint and simple harmony.
				See bottom of file for a key to the field contents.
LIMITATIONS:	Unable to differentiate between enharmonic equivalents.
*/


class Note
{
	/* FIELDS ******************************************************************/


	private int duration;	// the rhythmic duration of the note
	private int tied;		// indicates whether or not the note is tied to any other notes
	private int pitch;		// the pitch of the note


	/* CONSTRUCTOR *************************************************************/


	Note(int dur, int tie, int pit)
	{
		duration = dur;
		tied = tie;
		pitch = pit;
	}


	/* PUBLIC METHODS **********************************************************/


	public int get_pitch()
	// Returns the pitch of the note
	{
		return pitch;
	}


	/***************************************************************************/


	public int get_duration()
	// Returns the rhythmic duration of the note
	{
		return duration;
	}


	/***************************************************************************/


	public int get_tied()
	// Returns information about whether the note is tied or not
	{
		return tied;
	}


	/***************************************************************************/


	public void tie_ahead()
	// Ties the note to the note ahead of it
	{
		tied = 3;
	}


	/***************************************************************************/


	public void tie_behind()
	// Ties the note to the note behind it
	{
		tied = 2;
	}
}


/*
KEY

duration:
	1 = sixteenth note
	2 = eighth note
	3 = dotted eighth note
	4 = quarter note
	6 = dotted quarter note
	8 = half note
	12 = dotted half note
	16 = whole note

tied:
	1 = this note is not tied to any other notes
	2 = this note is tied to the preceding note only
	3 = this note is tied to the subsequent note only
	4 = this note is tied to both the preceding and subsequent notes

pitch:
	0 = this note is a rest
	1 = this note is a C -2
	2 = this note is a C#/Db -2
	3 = this note is a D -2
	4 = this note is a D#/Eb -2
	5 = this note is a E -2
	6 = this note is a F -2
	7 = this note is a F#/Gb -2
	8 = this note is a G -2
	9 = this note is a G#/Ab -2
	10 = this note is a A -2
	11 = this note is a A#/Bb -2
	12 = this note is a B -2
	13 = this note is a C -1
	14 = this note is a C#/Db -1
	15 = this note is a D -1
	16 = this note is a D#/Eb -1
	17 = this note is a E -1
	18 = this note is a F -1
	19 = this note is a F#/Gb -1
	20 = this note is a G -1
	21 = this note is a G#/Ab -1
	22 = this note is a A -1
	23 = this note is a A#/Bb -1
	24 = this note is a B -1
	25 = this note is a C1 (MIDDLE C)
	26 = this note is a C#/Db1
	27 = this note is a D1
	28 = this note is a D#/Eb1
	29 = this note is a E1
	30 = this note is a F1
	31 = this note is a F#/Gb1
	32 = this note is a G1
	33 = this note is a G#/Ab1
	34 = this note is a A1
	35 = this note is a A#/Bb1
	36 = this note is a B1
	37 = this note is a C2
	38 = this note is a C#/Db2
	39 = this note is a D2
	40 = this note is a D#/Eb2
	41 = this note is a E2
	42 = this note is a F2
	43 = this note is a F#/Gb2
	44 = this note is a G2
	45 = this note is a G#/Ab2
	46 = this note is a A2
	47 = this note is a A#/Bb2
	48 = this note is a B2
	49 = this note is a C3
	50 = this note is a C#/Db3
	51 = this note is a D3
	52 = this note is a D#/Eb3
	53 = this note is a E3
	54 = this note is a F3
	55 = this note is a F#/Gb3
	56 = this note is a G3
	57 = this note is a G#/Ab3
	58 = this note is a A3
	59 = this note is a A#/Bb3
	60 = this note is a B3
*/