/*
SpeciesChecker
by Cory McKay

CLASS NAME:		Voice
CLASS TYPE:		Fundamental data structure
DESCRIPTION:	Objects of this represent the contents of a single voice in a piece of music.
				Appropriate for use in species counterpoint and simple harmony.
				See bottom of file for a key to the field contents.
*/


class Voice
{
	/* FIELDS ******************************************************************/


	private int type_of_line;	// what type of line this voice corresponds to
	private int clef;			// what type of clef is used to display the contents of the voice
	private Note[] note;		// the notes in the line, organized by sixteenth note rhythmic units


	/* CONSTRUCTOR *************************************************************/


	Voice (int typ, int cle, int number_of_sixteenths)
	// Automatically sets all notes to null, since they are empty at this stage
	{
		type_of_line = typ;
		clef = cle;

		note = new Note[number_of_sixteenths];
		for(int i=0; i < number_of_sixteenths; i++)
			note[i] = null;
	}


	/* PUBLIC METHODS **********************************************************/


	public int get_type_of_line()
	// Returns the type of composition that the voice corresponds to
	{
		return type_of_line;
	}


	/***************************************************************************/


	public int get_clef()
	// Returns the type of clef that is used to display the contents of the voice
	{
		return clef;
	}


	/***************************************************************************/


	public Note get_note(int sixteenth_note_index)
	// Returns the note that is struck on the given sixteenth note rhythmic unit
	{
		return note[sixteenth_note_index];
	}


	/***************************************************************************/


	public void insert_note(Note to_be_inserted, int sixteenth_note_index)
	// Inserts the given noe into the given sixteenth note rhythmic unit
	// NOTE: this note replaces any note that was previously at that location
	{
		note[sixteenth_note_index] = to_be_inserted;
	}
}


/*
KEY

type_of_line
	0 = cantus firmus
	1 = first species above cantus
	2 = first species below cantus
	3 = second species above cantus
	4 = second species below cantus
	5 = third species above cantus
	6 = third species below cantus
	7 = fourth species above cantus
	8 = fourth species below cantus
	9 = fifth species above cantus
	10 = fifth species below cantus
	11 = bass in four-part composition
	12 = tenor in four-part composition
	13 = alto in four-part composition
	14 = soprano in four-part composition

clef
	1 = bass
	2 = C
	3 = alto

note[i]
	null = no note is struck on rhythmic unit i
	pointer to a Note = the note that is struck on rhythmic unit i
*/